package net.sf.basedb.clients.web.extensions.list;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import net.sf.basedb.clients.web.util.HTML;
import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.SyncFilter.SourceItemTransform;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.core.snapshot.AnnotationSnapshot;
import net.sf.basedb.core.snapshot.AnnotationTypeFilter;
import net.sf.basedb.util.Enumeration;
import net.sf.basedb.util.filter.Filter;

/**
	Related item column that load annotation values from the parent or child items.
	@author nicklas
	@since 3.16, 3.17
*/
public class RelatedItemAnnotationColumn 
	extends RelatedItemColumn
{

	private final AnnotationType at;
	private final Filter<AnnotationSnapshot> atFilter;
	
	RelatedItemAnnotationColumn(int index, String id, Item sourceType, Item targetType, SourceItemTransform direction, Restriction targetRestriction, RelatedItemHelper helper, AnnotationType at)
	{
		super(index, id, sourceType, targetType, direction, targetRestriction, helper);
		this.at = at;
		this.atFilter = new AnnotationTypeFilter(at);
		if (at.isEnumeration())
		{
			Enumeration<String, String> annotationEnum = new Enumeration<String, String>();
			annotationEnum.add("", "-none-");
			List<?> values = at.getValues();
			for (Object value : values)
			{
				String encoded = HTML.encodeTags(value.toString());
				annotationEnum.add(encoded, encoded);
			}
			setEnumeration(annotationEnum);
		}
	}
	
	@Override
	public Object getValue(RelatedItemHelper helper, Annotatable item) 
	{
		Set<Annotatable> items = getRelatedItems(item);
		if (items.size() == 0) return null;
		
		List<Object> values = new ArrayList<>();
		for (Annotatable p : items)
		{
			for (AnnotationSnapshot ss : helper.manager.findAnnotations(helper.dc, p, atFilter, true, false))
			{
				values.addAll(ss.getActualValues());
			}
		}
		return values.size() == 0 ? null : values;
	}

}
