package net.sf.basedb.clients.web.extensions.login;

import net.sf.basedb.clients.web.extensions.AbstractJspActionFactory;
import net.sf.basedb.util.extensions.InvokationContext;

/**
	A version of the default login form that can be 
	installed as an extension. This extension should be
	disabled by default. It should be enabled only to provide
	regular password login functionality after installing 
	another login manager.
	
	@author nicklas
	@since 3.14
*/
public class PasswordLoginFormFactory 
	extends AbstractJspActionFactory<LoginFormAction>
	implements LoginFormAction
{
	
	
	/**
		The default password login form.
	*/
	public static final LoginFormAction INSTANCE = new PasswordLoginFormFactory();
	
	public PasswordLoginFormFactory()
	{}

	/**
		@return A single {@link LoginFormBean} instance
	*/
	@Override
	public LoginFormAction[] getActions(InvokationContext<? super LoginFormAction> context)
	{
		String clientId = context.getClientContext().getSessionControl().getExternalClientId();
		return new LoginFormAction[] { this };
	}

	@Override
	public String getId() 
	{
		return "password";
	}

	@Override
	public String getDisplayName() 
	{
		return "Password";
	}

	@Override
	public String getHelp() 
	{
		return null;
	}

	@Override
	public FieldInfo getLoginField() 
	{
		return FieldInfo.DEFAULT_LOGIN;
	}

	@Override
	public FieldInfo getPasswordField() 
	{
		return FieldInfo.DEFAULT_PASSWORD;
	}

	@Override
	public boolean rememberLastLogin() 
	{
		return true;
	}

}
