/*
	$Id$

	Copyright (C) 2014 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.servlet;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.basedb.clients.web.extensions.skin.ImageRemapper;
import net.sf.basedb.clients.web.extensions.skin.ImageRemapperUtil;

/**
	This is a filter that remaps request to images to some other image.
	See {@link ImageRemapper}. The remap is done internally on the
	server and is not visible for the web client. It is possible to
	set a cache control header to control the time images are cached.
	
	@author Nicklas
	@since 3.4
*/
public class ImageRemapFilter 
	implements Filter
{

	private String cacheControl = null;
	private int lengthOfContextPath;
	private String contextPath;
	
	public ImageRemapFilter() 
	{}
	
	@Override
	public void init(FilterConfig config) 
		throws ServletException 
	{
		cacheControl = config.getInitParameter("cache-control");
		contextPath = config.getServletContext().getContextPath();
		lengthOfContextPath = contextPath.length();
	}

	@Override
	public void destroy()
	{}

	@Override
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) 
		throws IOException, ServletException 
	{
		if (cacheControl != null)
		{
			HttpServletResponse resp = (HttpServletResponse)response;
			resp.setHeader("Cache-Control", cacheControl);
		}
		
		ImageRemapper remapper = ImageRemapperUtil.getCurrentMapper();
		if (remapper != null)
		{
			HttpServletRequest req = (HttpServletRequest)request;			
			String src = req.getRequestURI().substring(lengthOfContextPath);
			String dest = remapper.get(src);
			if (dest != null)
			{
				request.getRequestDispatcher(dest).forward(request, response);
				return;
			}
		}
		
		chain.doFilter(request, response);
	}

}


