/**
	$Id: ItemProxy.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

/**

	@author nicklas
	@version 2.2
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (Fri, 14 Sep 2012) $
*/
public class ItemProxy
	implements Comparable<ItemProxy>
{

	private final Item type;
	private final int id;
	
	public ItemProxy(int id, Item type)
	{
		this.id = id;
		this.type = type;
	}
	
	/*
		From the Comparable interface
		-------------------------------------------
	*/
	@Override
	public int compareTo(ItemProxy o)
	{
		return this.type == o.type ? this.id - o.id : this.type.name().compareTo(o.type.name());
	}
	// -------------------------------------------

	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return type.toString()+"["+id+"]";
	}
	/**
		Check if this item is equal to another item. They are considered 
		to be equal if they are of the same type and have the same id.
	*/
	@Override
	public final boolean equals(Object o)
	{
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		ItemProxy other = (ItemProxy)o;
		return other.id == this.id && other.type == this.type;
	}
	/**
		The hash code is calculated from the {@link Item#getValue()}
		and the id.
	*/
	@Override
	public final int hashCode()
	{
		return 29 * type.getValue() + 7 * id;
	}
	// -------------------------------------------
		
	public int getId()
	{
		return id;
	}
	
	public Item getType()
	{
		return type;
	}
	
	public BasicItem getItem(DbControl dc)
		throws PermissionDeniedException, ItemNotFoundException, BaseException
	{
		return type.getById(dc, id);
	}
	
}
