/*
	$Id: LongParameterValueData.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.List;
import java.util.ArrayList;

/**
	Long parameter value.

	@author Nicklas, Samuel
	@version 2.0
	@see <a href="../../../../../../html/developer/api/data_api.parameters.html">Developer documentation: Parameters</a>
	@hibernate.subclass discriminator-value="2"
	@base.modified $Date: 2005-07-14 09:57:11 +0200 (to, 14 jul 2005) $
*/
public class LongParameterValueData
	extends ParameterValueData<Long>
{
	public LongParameterValueData()
	{}

	public LongParameterValueData(Long... values)
	{
		super(values);
	}	
	
	
	/**
		@return 2 (Type.LONG)
		@since 2.16
	*/
	@Override
	public int getValueType()
	{
		return 2;
	}

	private List<Long> values;
	/**
		@hibernate.bag table="`LongValues`" lazy="true" cascade="all"
		@hibernate.collection-key column="`id`"
		@hibernate.collection-element column="`value`" type="long" not-null="true"
	*/
	@Override
	public List<Long> getValues()
	{
		if (values == null)
		{
			values = new ArrayList<Long>();
		}
		return values;
	}
	@Override
	void setValues(List<Long> values)
	{
		this.values = values;
	}

	@Override
	public String toString()
	{
		return values.toString();
	}
}
