/*
  $Id: NewsData.java 7392 2017-06-09 07:33:19Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;

/**
	This class holds information about news items.
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.News
	@see <a href="../../../../../../html/developer/api/data_api.misc.html">Developer documentation: Other classes</a>
	@hibernate.class table="`News`" lazy="false"
 */
public class NewsData
	extends BasicData
	implements NameableData, RemovableData, RegisteredData
{
	public NewsData()
	{}

	/*
		From the NameableData interface
		-------------------------------------------
	*/	
	private String name;
	@Override
	public String getName()
	{
		return name;
	}

	@Override
	public void setName(String name)
	{
		this.name = name;
	}

	private String description;
	@Override
	public String getDescription()
	{
		return description;
	}

	@Override
	public void setDescription(String description)
	{
		this.description = description;
	}
	// -------------------------------------------

	/*
		From the RemovableData interface
		-------------------------------------------
	*/
	private Integer removedBy;
	@Override
	public Integer getRemovedBy()
	{
		return removedBy;
	}
	@Override
	public void setRemovedBy(Integer removedBy)
	{
		this.removedBy = removedBy;
	}
	// -------------------------------------------
	/*
		From the RegisteredData interface
		-------------------------------------------
	*/
	private Date entryDate;
	@Override
	public Date getEntryDate()
	{
		return entryDate;
	}
	public void setEntryDate(Date entryDate)
	{
		this.entryDate = entryDate;
	}
	// -------------------------------------------

	private Date startDate;
	/**
		Get the first day this news item should be shown.
		@hibernate.property column="`start_date`" type="date" not-null="true"
	*/
	public Date getStartDate()
	{
		return startDate;
	}
	public void setStartDate(Date startDate)
	{
		this.startDate = startDate;
	}

	private Date newsDate;
	/**
		Get the news date of this news item.
		@hibernate.property column="`news_date`" type="date" not-null="true"
	*/
	public Date getNewsDate()
	{
		return newsDate;
	}
	public void setNewsDate(Date newsDate)
	{
		this.newsDate = newsDate;
	}

	private Date endDate;
	/**
		Get the last day this news item should be shown.
		@hibernate.property column="`end_date`" type="date"
	*/
	public Date getEndDate()
	{
		return endDate;
	}
	public void setEndDate(Date endDate)
	{
		this.endDate = endDate;
	}
	
	private boolean sticky;
	/**
		Sticky news should be sorted before non-sticky news regardless of
		the date.
		@since 3.12
		@hibernate.property column="`sticky`" type="boolean" not-null="true" 
	*/
	public boolean isSticky()
	{
		return sticky;
	}
	public void setSticky(boolean sticky)
	{
		this.sticky = sticky;
	}

}
