/**
	$Id $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.plugin;

import java.io.FilterInputStream;
import java.io.InputStream;
import java.nio.charset.Charset;

/**
	An input stream used for input to import plug-ins. The input stream
	is a regular {@link InputStream} but provides methods for accessing
	metadata about the original source the stream is originating from.
	<p>
	This class is intended to be subclassed by implementations more 
	suitable for the type of import. For example the 
	{@link net.sf.basedb.util.FileImportInputStream} for importing from 
	files on the BASE file system.
	
	@author nicklas
	@version 2.9
	@base.modified $Date: 2008-09-16 08:31:13 +0200 (Tue, 16 Sep 2008) $
*/
public class ImportInputStream 
	extends FilterInputStream 
{

	/**
		Create a new import stream.
		@param in The source stream to read from
	*/
	public ImportInputStream(InputStream in)
	{
		super(in);
	}
	
	/**
		Get the length of the stream in number of bytes. The
		default implementation returns -1.
		
		@return The number of bytes, or -1 if not known
	*/
	public long getLength()
	{
		return -1;
	}
	
	/**
		Get the MIME type of the source.
		@return The MIME type of null if not known
	*/
	public String getMimeType()
	{
		return null;
	}
	
	/**
		Get the character set used by the source if it is a text-file.
		@return The character set, or null if not known
		@see Charset#forName(String)
	*/
	public String getCharacterSet()
	{
		return null;
	}
	
	/**
		Get the name of the source.
		@return The name, or null if not known
	*/
	public String getFilename()
	{
		return null;
	}

}
