/*
	$Id: MeanExpression.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Johan Enell

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;
import java.util.Collections;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.util.EqualsHelper;

/**
	Calculates the arithmetic mean of an expression: AVG(e)

	@author Nicklas
	@version 2.0
	@see Aggregations#mean(Expression)
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (Fri, 14 Sep 2012) $
*/
public class MeanExpression
	implements Expression 
{
	private final Expression e;

	MeanExpression(Expression e)
	{
		assert e != null : "e == null";
		this.e = e;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	@Override
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		return "AVG(" + e.toQl(query, dc) + ")";
	}
	/**
		@return The expression to find the mean for
	*/
	@Override
	public Collection<? extends QueryElement> getChildren()
	{
		return Collections.singleton(e);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return "AVG(" + e + ")";
	}
	/**
		This expression is equal to another MeanExpression if their 
		child expressions are equal.
	*/
	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		MeanExpression o = (MeanExpression)other;
		return EqualsHelper.equals(e, o.e);
	}
	@Override
	public int hashCode()
	{
		return EqualsHelper.hashCode(e) + getClass().hashCode();
	}
	// -------------------------------------------
}

