/**
	$Id$

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions.manager;

import net.sf.basedb.util.extensions.xml.PluginInfo;

/**
	Key implementation used to identify plug-in definitions.
	Since a plug-in can be identified by it's class name,
	we use that as the key for equality.

	@author Nicklas
	@since 3.0
	@base.modified $Date$
	@see PluginInfo
*/
public class PluginInfoKey
	implements ObjectKey<PluginInfo>
{
	
	private final String className;
	
	/**
		Create a new key for a plug-in with the given class name.
		@param className The class name (required)
	*/
	public PluginInfoKey(String className)
	{
		if (className == null) throw new NullPointerException("className");
		this.className = className;
	}

	/**
		Create a new key for an plug-in definition.
		@param info Information about the plug-in (required)
	*/
	public PluginInfoKey(PluginInfo info)
	{
		this(info.getClassName());
	}
	
	/*
		From the ObjectKey interface
		----------------------------
	*/
	@Override
	public String toDescription()
	{
		return "Plugin '" + className + "'";
	}
	// -----------------------------
	
	/*
		From the Object class
		---------------------
	*/
	@Override
	public int hashCode()
	{
		return className.hashCode();
	}

	@Override
	public boolean equals(Object obj)
	{
		if (this == obj) return true;
		if (obj == null || obj.getClass() != this.getClass()) return false;
		PluginInfoKey other = (PluginInfoKey)obj;
		return other.className.equals(this.className);
	}

	@Override
	public String toString()
	{
		return "PluginInfoKey@" + System.identityHashCode(this) + "[" + className + "]";
	}
	// -----------------------
	
	
}
