/**
	$Id: Formatter.java 5432 2010-09-29 09:21:45Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.formatter;

/**
 	A <code>Formatter</code> formats an object to a string suitable for
 	output in a client application. The formatter may optionally also do
 	the reverse, ie. parse a string to an object.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2010-09-29 11:21:45 +0200 (Wed, 29 Sep 2010) $
*/
public interface Formatter<T>
{

	/**
		Format a value by converting it to a string.
		@param value The value to format
		@return A formatted string
	*/
	public String format(T value);

	/**
		Parse a string and return a value of the correct type. A null
		input string should (unless otherwise documented by a specific
		formatter implementation) be allowed and may return null or
		any other "default" value.
		
		@param value The string to parse
		@return An object
		@since 2.2
	*/
	public T parseString(String value);
	
}
