package net.sf.basedb.plugins.batchimport;

import net.sf.basedb.core.BioMaterialEventSource;
import net.sf.basedb.core.MeasuredBioMaterial;
import net.sf.basedb.util.parser.FlatFileParser.Data;
import net.sf.basedb.util.parser.Mapper;

/**
	A special mapper for "Used quantity" values. It allows a special case where "ALL" is specified
	to use all the remaining quantity from the parent item.
	@since 3.19.1
*/
public class UsedQuantityMapper
{

	public static UsedQuantityMapper create(Mapper mapper)
	{
		return mapper == null ? null : new UsedQuantityMapper(mapper);
	}
	
	private final Mapper mapper;
	public UsedQuantityMapper(Mapper mapper)
	{
		this.mapper = mapper;
	}
	
	/**
		Get the used quantity from the parent item. 
	*/
	public Float getUsedQuantity(Data data, BioMaterialEventSource event)
	{
		Float used = null;
		String v = mapper.getString(data);
		if ("ALL".equals(v))
		{
			used = event.getUsedQuantity();
			MeasuredBioMaterial p = (MeasuredBioMaterial)event.getBioMaterial();
			Float remain = p.getRemainingQuantity();
			if (used == null)
			{
				used = remain;
			}
			else if (remain != null)
			{
				used += remain;
			}
		}
		else
		{
			used = mapper.getFloat(data);
		}
		return used;
	}
	
}
