import java.util.Date;
import java.util.Set;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemProxy;
import net.sf.basedb.core.MimeType;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.ItemResultList;

/*
 $Id$

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */

public class TestMimeType
{

	static private boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}
	static boolean test_all()
	{
		write("++Testing mime types");
		write_header();
		// Standard tests: create, load, list
		int id1 = test_create(true);
		int id2 = test_create(false);
		test_load(id1);
		test_list();

		// Extra tests:

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id1);
		test_delete(id2);
		write("++Testing mime types "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(boolean testAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.MIMETYPE)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			MimeType mt = MimeType.getNew(dc);
			if (testAll)
			{
				mt.setName("x-test/mimetype");
				mt.setDescription("Added at: "+new Date());
				mt.setExtension("x-test");
				mt.setAutoCompress(true);
			}
			dc.saveItem(mt);
			dc.commit();
			id = mt.getId();
			write_item(0, mt);
			write("--Create mime type OK");
		}
		catch (Throwable ex)
		{
			write("--Create mime type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl bc = null;
		try
		{
			bc = TestUtil.getDbControl();
			MimeType mt = MimeType.getById(bc, id);
			write_item(0, mt);
			write("--Load mime type OK");
		}
		catch (Throwable ex)
		{
			write("--Load mime type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (bc != null) bc.close();
		}
	}

	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<MimeType> l= MimeType.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List mime types OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List mime types FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			MimeType mt = MimeType.getById(dc, id);
			dc.deleteItem(mt);
			Set<ItemProxy> using = mt.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(mt + " is used by " + using);
			}
			dc.commit();
			write("--Delete mime type OK");
		}
		catch (Throwable ex)
		{
			write("--Delete mime type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tExtension\tAuto-compress");
			write("-- \t-- \t--------- \t-----------\t---------\t-------------");
		}
	}
	static void write(String message)
	{
		System.out.println(message);
	}
	static void write_item(int i, MimeType mt)
	{
		if (!TestUtil.getSilent())
		{
			System.out.println(i+":\t"+mt.getId()+"\t"+mt.getName()+"\t"+mt.getDescription()+"\t"+
				mt.getExtension() + "\t" + mt.getAutoCompress());
		}
	}
}
