<%-- $Id: list_annotations.jsp 7954 2021-05-12 13:27:22Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.AnnotationType"
	import="net.sf.basedb.core.Protocol"
	import="net.sf.basedb.core.AnnotationSet"
	import="net.sf.basedb.core.Annotation"
	import="net.sf.basedb.core.Annotatable"
	import="net.sf.basedb.core.AnnotatableProxy"
	import="net.sf.basedb.core.Unit"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.Subtypable"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.BioPlate"
	import="net.sf.basedb.core.BioPlateType"
	import="net.sf.basedb.core.RawDataType"
	import="net.sf.basedb.core.RawBioAssay"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.core.snapshot.AnnotationSetSnapshot"
	import="net.sf.basedb.core.snapshot.AnnotationSnapshot"
	import="net.sf.basedb.core.snapshot.SnapshotManager"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.NameableComparator"
	import="net.sf.basedb.util.units.UnitConverter"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.util.formatter.PrefixSuffixFormatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.util.Values"
	import="java.util.Collections"
	import="java.util.ArrayList"
	import="java.util.List"
	import="java.util.Set"
	import="java.util.HashSet"
	import="java.util.TreeSet"
	import="java.util.Map"
	import="java.util.HashMap"
	import="java.util.Date"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>

<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final Item itemType = Item.valueOf(request.getParameter("item_type"));
final int itemId = Values.getInt(request.getParameter("item_id"));
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl(":List annotations");

Set<AnnotationType> annotationTypes = null;
Set<AnnotationType> protocolParameters = null;
Map<AnnotationType, AnnotationSnapshot> existing = null;

try
{
	final Annotatable item = (Annotatable)itemType.getById(dc, itemId);
	final boolean writePermission = item.hasPermission(Permission.RESTRICTED_WRITE);
	final SnapshotManager manager = new SnapshotManager();
	
	ItemQuery<AnnotationType> annotationTypeQuery = null;
	String message = null;
	boolean isProxy = item instanceof AnnotatableProxy;

	// Load all annotation types that are possible for this item type
	if (isProxy)
	{
		AnnotatableProxy proxy = (AnnotatableProxy)item;
		annotationTypeQuery = Base.getAnnotationTypesQuery(proxy);
		message = proxy.getAnnotationMessage();
	}
	else
	{
		annotationTypeQuery = Base.getAnnotationTypesQuery(itemType, false);
	}
	annotationTypes = new TreeSet<AnnotationType>(new NameableComparator<>(false));
	annotationTypes.addAll(annotationTypeQuery.list(dc));

	final AnnotationSetSnapshot snapshot = item.isAnnotated() ? 
			manager.getSnapshot(dc, item.getAnnotationSet().getId()) : null;
	List<AnnotationSnapshot> inheritedAnnotations = null;
	if (snapshot != null)
	{
		// Load the existing primary and inherited annotations
		existing = new HashMap<AnnotationType, AnnotationSnapshot>();
		inheritedAnnotations = new ArrayList<AnnotationSnapshot>();
		for (AnnotationSnapshot a : manager.findAnnotations(dc, snapshot, null, true))
		{
			AnnotationType at = a.getAnnotationType(dc);
			if (a.getSource() == Annotation.Source.PRIMARY)
			{
				existing.put(a.getAnnotationType(dc), a);
			}
			else
			{
				inheritedAnnotations.add(a);
			}
		}
		Collections.sort(inheritedAnnotations, AnnotationSnapshot.sortByAnnotationType(dc));
		annotationTypes.addAll(existing.keySet());
	}
	
	// Load the possible protocol parameters
	Protocol protocol = null;
	boolean readProtocol = true;
	try
	{
		protocol = item.getProtocol();
	}
	catch (PermissionDeniedException ex)
	{
		readProtocol = false;
	}
	ItemQuery<AnnotationType> parameterQuery = Base.getProtocolParametersQuery(protocol);
	if (parameterQuery != null)
	{
		protocolParameters = new HashSet<AnnotationType>(parameterQuery.list(dc));
		annotationTypes.removeAll(protocolParameters);
	}
	// Load the current subtype
	// NOTE! User may have selected a different subtype in the form than what is
	// currently saved so if a subtype_id is sent in the request we use that
	String selectedCategoryName = null;
	try
	{
		ItemSubtype subtype = null;
		if (item instanceof Subtypable)
		{
			subtype = ((Subtypable)item).getItemSubtype();
		}
		else if (item instanceof BioPlate)
		{
			BioPlateType bpt = ((BioPlate)item).getBioPlateType();
			subtype = bpt.getItemSubtype();
			if (subtype == null) selectedCategoryName = bpt.getName();
		}
		else if (item instanceof RawBioAssay)
		{
			RawDataType rdt = ((RawBioAssay)item).getRawDataType();
			selectedCategoryName = rdt.getName();
		}
		if (subtype != null) selectedCategoryName = subtype.getName();
	}
	catch (PermissionDeniedException ex)
	{}
	Set<AnnotationType> annotationTypesInCategory = new HashSet<AnnotationType>();
	if (selectedCategoryName != null)
	{
		ItemQuery<AnnotationType> query = AnnotationType.getQuery(item.getType());
		query.setIncludes(Include.ALL);
		query.join(Hql.innerJoin("categories", "ct"));
		query.restrict(Restrictions.eq(Hql.property("ct", "name"), Expressions.string(selectedCategoryName)));
		annotationTypesInCategory.addAll(query.list(dc));
	}
	Formatter<Date> dateTimeFormatter = FormatterFactory.getDateTimeFormatter(sc);
	%>
	<base:page type="iframe" id="list-annotations">
	<base:head scripts="~list.js" styles="table.css">
	<style>
		#annotations-data.all
		{}
		
		#annotations-data.values .novalues
		{
			display: none;
		}
		
		#annotations-data.category .nocategory
		{
			display: none;
		}
		
		.edit-annotation
		{
		 	float: right;
		}
	</style>
	</base:head>
	<base:body>
		<%
		if (message != null)
		{
			%>
			<div class="messagecontainer note"><%=message%></div>
			<%
		}
		%>
		<base:section
			id="primaryAnnotations"
			title="<%="Primary annotations (" + annotationTypes.size() + ")"%>"
			>
			<%
			int allWithValues = 0;
			int allOfCategory = 0;
			if (annotationTypes.size() == 0)
			{
				%>
				<div class="messagecontainer note">
				No annotation types has been defined for this type of item 
				(or you don't have permission to view them).
				</div>
				<%
			}
			else
			{
				%>
				<tbl:table
					id="annotations"
					columns="all"
					stickyheaders="<%=null%>"
					>
					<tbl:columndef 
						id="annotation"
						title="Annotation"
					/>
					<tbl:columndef 
						id="values"
						title="Values"
					/>
					<tbl:columndef
						id="note"
						title="Note"
					/>
					<tbl:columndef
						id="lastModified"
						title="Last modified"
						formatter="<%=dateTimeFormatter%>"
					/>
					<tbl:columndef 
						id="description"
						title="Description"
					/>
					<tbl:data id="annotations-data">
						<tbl:headers>
							<tbl:headerrow>
								<tbl:header colspan="5" subclass="bottomborder" style="padding: 5px;">
									<b>View:</b> 
									<%
									if (selectedCategoryName != null)
									{
										%>
										<label><input type="radio" name="view" id="view_category" value="category" checked>All of <i><%=HTML.encodeTags(selectedCategoryName)%></i> category</label> (<span id="allOfCategory">?</span>)
										<%
									}
									%>
									<label><input type="radio" name="view" id="view_values" value="values" <%=selectedCategoryName == null ? "checked" : ""%>>All with values</label> (<span id="allWithValues">?</span>)
									<label><input type="radio" name="view" id="view_all" value="all">All annotations</label> (<%=annotationTypes.size()%>)
								</tbl:header>
							</tbl:headerrow>
							<tbl:headerrow>
								<tbl:columnheaders />
							</tbl:headerrow>
						</tbl:headers>
						<tbl:rows>
						<%
						for (AnnotationType at : annotationTypes)
						{
							// Permission for creating a new annotation
							boolean annotatePermission = writePermission & at.hasPermission(Permission.USE);
							AnnotationSnapshot a = existing != null ? existing.get(at) : null;
							List<?> values = null;
							List<?> defaultValues = null;
							Formatter<Object> formatter = null;
							boolean projectSpecific = false;
							if (a != null)
							{
								Annotation ann = a.getThisAnnotation(dc);
								// Permission on this annotation
								annotatePermission = ann.hasPermission(Permission.WRITE);
								values = a.getActualValues(ann.getUnitConverter(null), ann.getValueType());
								formatter = FormatterFactory.getAnnotationFormatter(sc, ann, null);
								
								projectSpecific = a.getProjectId() != 0;
								if (ann.isOverridingDefault())
								{
									Annotation defAnn = ann.getDefaultAnnotation();
									defaultValues = defAnn.getValues();
								}
							}
							if (values != null || !at.isRemoved())
							{
								String rowClass = "";
								if (values == null || values.size() == 0) 
								{
									rowClass += " novalues";
								}
								else
								{
									allWithValues++;
								}
								if (!annotationTypesInCategory.contains(at)) 
								{
									rowClass += " nocategory";
								}
								else
								{
									allOfCategory++;
								}
								%>
								<tbl:row subclass="<%=rowClass%>">
									<tbl:cell column="annotation"><base:icon 
										image="deleted.png" 
										tooltip="This item has been scheduled for deletion" 
										visible="<%=at.isRemoved()%>"
										/><%=Base.getLinkedName(ID, at, false, true)%></tbl:cell>
									<tbl:cell column="values" clazz="<%=projectSpecific && defaultValues != null ? "cell ps-annotation" : "cell" %>">
										<%=values == null || values.size() == 0 ? "<i>- no values -</i>" : Values.getString(values, ", ", true, formatter)%>
										<base:icon image="edit.png" subclass="edit-annotation auto-init link" 
											data-auto-init="edit-annotation" 
											data-annotation="<%=a!=null ? a.getThisAnnotationId() : 0 %>"
											data-annotation-type="<%=at.getId()%>"
											data-item-type="<%=itemType.name()%>" 
											data-item-id="<%=itemId %>"
											tooltip="Modify the values of this annotation"
											visible="<%=annotatePermission %>"
										/>
									</tbl:cell>
									<tbl:cell column="note"> 
										<%
										if (projectSpecific)
										{
											%>
											<%=defaultValues == null || defaultValues.size() == 0 ? "No default value" : "Default value: " + Values.getString(defaultValues, ", ", true, formatter)%>
											<%
										}
										%>
									</tbl:cell>
									<tbl:cell column="lastModified" value="<%=a == null ? null : a.getThisLastUpdate() %>" />
									<tbl:cell column="description"><%=HTML.encodeTags(at.getDescription())%></tbl:cell>
								</tbl:row>
								<%
							}
						}
						%>
						</tbl:rows>
					</tbl:data>
				</tbl:table>
				<%
			}
			%>
			<div id="annotations-stat" class="datacontainer"
				data-all-with-values="<%=allWithValues %>"
				data-all-of-category="<%=allOfCategory%>"
			></div>
		</base:section>

		<%
		if (protocolParameters != null)
		{
			%>
			<base:section 
				id="parameterSection" 
				title="<%="Protocol parameters for " + HTML.encodeTags(protocol.getName()) + 
					" (" + protocolParameters.size() + ")"%>"
				>
				<%
				if (protocolParameters.size() == 0)
				{
					%>
					<div class="messagecontainer note">
					No protocol parameters has been defined for the protocol 
					<%=Base.getLinkedName(ID, protocol, !readProtocol, true)%> 
					(or you don't have permission to view them).
					</div>
					<%
				}
				else
				{
					%>
					<tbl:table
						id="parameters"
						columns="all"
						>
						<tbl:columndef 
							id="parameter"
							title="Parameter"
						/>
						<tbl:columndef 
							id="values"
							title="Values"
						/>
						<tbl:columndef
							id="note"
							title="Note"
						/>
						<tbl:columndef
							id="lastModified"
							title="Last modified"
							formatter="<%=dateTimeFormatter%>"
						/>
						<tbl:columndef 
							id="description"
							title="Description"
						/>
						<tbl:data>
							<tbl:headers>
								<tbl:headerrow>
									<tbl:columnheaders />
								</tbl:headerrow>
							</tbl:headers>
							<tbl:rows>
							<%
							for (AnnotationType at : protocolParameters)
							{
								// Permission for creating a new annotation
								boolean annotatePermission = writePermission & at.hasPermission(Permission.USE);
								AnnotationSnapshot a = existing != null ? existing.get(at) : null;
								Formatter<Object> formatter = null;
								List<?> values = null;
								List<?> defaultValues = null;
								boolean projectSpecific = false;
								if (a != null)
								{
									Annotation ann = a.getThisAnnotation(dc);
									// Permission on this annotation
									annotatePermission = ann.hasPermission(Permission.WRITE);
									values = a.getActualValues(ann.getUnitConverter(null), ann.getValueType());
									formatter = FormatterFactory.getAnnotationFormatter(sc, ann, null);
									
									projectSpecific = a.getProjectId() != 0;
									if (ann.isOverridingDefault())
									{
										Annotation defAnn = ann.getDefaultAnnotation();
										defaultValues = defAnn.getValues();
									}
								}
								if (values != null || !at.isRemoved())
								{
									%>
									<tbl:row>
										<tbl:cell column="parameter"><base:icon 
											image="deleted.png" 
											tooltip="This item has been scheduled for deletion" 
											visible="<%=at.isRemoved()%>"
										/><%=Base.getLinkedName(ID, at, false, true)%></tbl:cell>
										<tbl:cell column="values" clazz="<%=projectSpecific && defaultValues != null ? "cell ps-annotation" : "cell" %>">
											<%=values == null || values.size() == 0 ? "<i>- no values -</i>" : Values.getString(values, ", ", true, formatter)%>
											<base:icon image="edit.png" subclass="edit-annotation auto-init link" 
												data-auto-init="edit-annotation" 
												data-annotation="<%=a!=null ? a.getThisAnnotationId() : 0 %>"
												data-annotation-type="<%=at.getId()%>"
												data-item-type="<%=itemType.name()%>" 
												data-item-id="<%=itemId %>"
												tooltip="Modify the values of this protocol parameter" 
												visible="<%=annotatePermission %>"
											/>
										</tbl:cell>
										<tbl:cell column="note"> 
											<%
											if (projectSpecific)
											{
												%>
												<%=defaultValues == null || defaultValues.size() == 0 ? "No default value" : "Default value: " + Values.getString(defaultValues, ", ", true, formatter)%>
												<%
											}
											%>
										</tbl:cell>
										<tbl:cell column="lastModified" value="<%=a == null ? null : a.getThisLastUpdate() %>" />
										<tbl:cell column="description"><%=HTML.encodeTags(at.getDescription())%></tbl:cell>
									</tbl:row>
									<%
								}
							}
							%>
							</tbl:rows>
						</tbl:data>
					</tbl:table>
					<%
				}
				%>
			</base:section>
			<%
		}
		if (!isProxy)
		{
			int numInherited = inheritedAnnotations == null ? 0 : inheritedAnnotations.size();
			%>
			<base:section 
				id="inherited"
				title="<%="Inherited annotations (" + numInherited + ")"%>"					
				>
				<%
				if (numInherited == 0)
				{
					%>
					<div class="messagecontainer note">
					No annotations are inherited by this item.
					(or you don't have permission to view them).
					</div>
					<%
				}
				else
				{
				%>
				<tbl:table
					id="inheritedAnnotations"
					columns="all"
					>
					<tbl:columndef 
						id="annotation"
						title="Annotation"
					/>
					<tbl:columndef 
						id="values"
						title="Values"
					/>
					<tbl:columndef 
						id="item"
						title="Parent item"
					/>
					<tbl:columndef 
						id="note"
						title="Note"
					/>
					<tbl:columndef
						id="lastModified"
						title="Last modified"
						formatter="<%=dateTimeFormatter %>"
					/>
					<tbl:columndef 
						id="description"
						title="Description"
					/>
					<tbl:data>
						<tbl:headers>
							<tbl:headerrow>
								<tbl:columnheaders />
							</tbl:headerrow>
						</tbl:headers>
						<tbl:rows>
						<%
						for (AnnotationSnapshot a : inheritedAnnotations)
						{
							boolean annotatePermission = a.hasPermission(dc, Permission.WRITE);
							AnnotationType at = a.getAnnotationType(dc);
							AnnotationSnapshot from = a.getInheritedFrom();
							
							// About this annotation
							String name = HTML.encodeTags(at.getName());
							String description = HTML.encodeTags(at.getDescription());
							Date lastModified = a.getThisLastUpdate();
							boolean isUpToDate = a.isUpToDate();
							
							// Which item are we inheriting from?
							Nameable fromItem = null;
							String fromType = null;
							boolean writeInherited = false;
							if (from != null)
							{
								fromType = from.getThisItemType().toString();
								if (lastModified == null)
								{
									lastModified = from.getThisLastUpdate();
								}
								try
								{
									fromItem = (Nameable)from.getThisItem(dc);
									writeInherited = fromItem.hasPermission(Permission.WRITE);
									if (fromItem instanceof Subtypable)
									{
										ItemSubtype subtype = ((Subtypable)fromItem).getItemSubtype();
										if (subtype != null) fromType = subtype.getName();
									}
								}
								catch (PermissionDeniedException ex)
								{}
							}
							
							// Values, units, etc.
							Formatter<Object> formatter = FormatterFactory.getTypeFormatter(sc, at.getValueType());
							Unit unit = a.getActualUnit(dc);
							UnitConverter converter = null;
							if (unit != null)
							{
								converter = unit.getUnitConverter(at.getDefaultUnit());
								formatter = unit.getFormatter(formatter);
							}
							List<?> values = a.getActualValues(converter, at.getValueType());
							List<?> defaultValues = null;
							boolean projectSpecific = a.getProjectId() != 0;
							if (projectSpecific)
							{
								Annotation ann = a.getThisAnnotation(dc);
								if (ann.isOverridingDefault())
								{
									Annotation defAnn = ann.getDefaultAnnotation();
									Annotation defFrom = defAnn.getInheritedFrom();
									defaultValues = defAnn.getSource() == Annotation.Source.CLONED ? defAnn.getValues() : defFrom.getValues();
								}
							}
							%>
							<tbl:row>
								<tbl:cell column="annotation"><%=Base.getLinkedName(ID, at, at == null, true)%></tbl:cell>
								<tbl:cell column="values" clazz="<%=projectSpecific && defaultValues != null ? "cell ps-annotation" : "cell" %>">
									<%=values == null || values.size() == 0 ? 
										"<i>- no values -</i>" : Values.getString(values, ", ", true, formatter)%>
									<base:icon image="<%=isUpToDate ? "edit.png" : "edit-outofsync.png"%>" 
										subclass="edit-annotation auto-init link" 
										data-auto-init="edit-annotation" 
										data-annotation="<%= a.getThisAnnotationId()%>"
										data-annotation-type="<%=at.getId()%>"
										data-item-type="<%=itemType.name()%>" 
										data-item-id="<%=itemId %>"
										tooltip="Modify the values of this annotation"
										visible="<%=annotatePermission %>"
									/>
								</tbl:cell>
								<tbl:cell column="item"><%=from == null ? "<i>- missing -</i>" : Base.getLinkedName(ID, fromItem, fromItem == null, true)%><%=fromItem != null ? " <span class=\"itemsubtype\">("+fromType+")</span>" : ""%></tbl:cell>
								<tbl:cell column="note">
									<%=a.getSource() == Annotation.Source.CLONED ? "Cloned" + (isUpToDate ? "; up-to-date" : "; modified") : "" %>
										<%
										if (projectSpecific && at.getProjectAnnotations())
										{
											%>
											<%=defaultValues == null || defaultValues.size() == 0 ? "No default value" : "Default value: " + Values.getString(defaultValues, ", ", true, formatter)%>
											<%
										}
										%>
								</tbl:cell>
								<tbl:cell column="lastModified" value="<%=lastModified %>"></tbl:cell>
								<tbl:cell column="description"><%=description%></tbl:cell>
							</tbl:row>
							<%
						}
						%>
						</tbl:rows>
					</tbl:data>
				</tbl:table>
				<%
			}
			%>
			</base:section>
			
			<%
		}
		%>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}

%>
