<%-- $Id: list_files.jsp 7954 2021-05-12 13:27:22Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.BasicItem"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Platform"
	import="net.sf.basedb.core.PlatformVariant"
	import="net.sf.basedb.core.PlatformFileType"
	import="net.sf.basedb.core.DataFileType"
	import="net.sf.basedb.core.Subtypable"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.FileSet"
	import="net.sf.basedb.core.FileSetMember"
	import="net.sf.basedb.core.File"
	import="net.sf.basedb.core.FileStoreEnabled"
	import="net.sf.basedb.core.UsableDataFileType"
	import="net.sf.basedb.core.ItemSubtypeFileType"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.util.NameableComparator"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.fileviewer.FileViewerContext"
	import="net.sf.basedb.clients.web.extensions.fileviewer.FileViewerUtil"
	import="net.sf.basedb.util.Values"
	import="java.util.Map"
	import="java.util.Set"
	import="java.util.HashSet"
	import="java.util.HashMap"
	import="java.util.List"
	import="java.util.LinkedList"
	import="java.util.Collection"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%!

private static class DataFile
{
	final DataFileType type;
	final FileSetMember member;
	final FileStoreEnabled parent;
	DataFile(DataFileType type, FileSetMember member, FileStoreEnabled parent)
	{
		this.type = type;
		this.member = member;
		this.parent = parent;
	}
}

%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final Item itemType = Item.valueOf(request.getParameter("item_type"));
final int itemId = Values.getInt(request.getParameter("item_id"));
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl(":List data files");
final ItemContext cc = sc.getCurrentContext(itemType);
final String root = request.getContextPath() + "/";

try
{
	final FileStoreEnabled item = (FileStoreEnabled)itemType.getById(dc, itemId);
	final FileSet fileSet = item.hasFileSet() ? item.getFileSet() : null;
	Collection<FileSet> parents = null;
	try
	{
		parents = item.getParentFileSets();
	}
	catch (Throwable t)
	{}
	
	Platform platform = null;
	PlatformVariant variant = null;
	ItemSubtype subtype = null;
	try
	{
		platform = item.getPlatform();
		variant = item.getVariant();
	}
	catch (Throwable t)
	{}
	if (item instanceof Subtypable)
	{
		try
		{
			subtype = ((Subtypable)item).getItemSubtype();
		}
		catch (Throwable t)
		{}
	}
	
	// Load member file types
	List<DataFile> members = new LinkedList<DataFile>();
	Set<DataFileType> existing = new HashSet<DataFileType>();
	if (fileSet != null)
	{
		ItemQuery<FileSetMember> query = fileSet.getMembers();
		query.order(Orders.asc(Hql.property("dataFileType.name")));
		query.order(Orders.asc(Hql.property("file.name")));
		for (FileSetMember member : query.list(dc))
		{
			DataFileType dft = null;
			try
			{
				dft = member.getDataFileType();
				existing.add(dft);
			}
			catch (Throwable t)
			{}
			members.add(new DataFile(dft, member, null));
		}
	}
	int numMembers = members.size();
	if (parents != null && !parents.isEmpty())
	{
		for (FileSet pFileSet : parents)
		{
			try
			{
				FileStoreEnabled  parent = pFileSet.getItem();
				for (FileSetMember pMember : pFileSet.getMembers().list(dc))
				{
					DataFileType dft = null;
					try
					{
						dft = pMember.getDataFileType();
					}
					catch (Throwable t)
					{}
					members.add(new DataFile(dft, pMember, parent));
				}
			}
			catch (Throwable t)
			{}
		}
	}
	int numParents = members.size() - numMembers;
	

	// Load platform file types
	Map<DataFileType, UsableDataFileType> usableFileTypes =
		new HashMap<DataFileType, UsableDataFileType>();
	if (platform != null)
	{
		ItemQuery<PlatformFileType> query = platform.getFileTypes(variant, variant == null);
		for (PlatformFileType pft : query.list(dc))
		{
			DataFileType dft = null;
			try
			{
				dft = pft.getDataFileType();
				usableFileTypes.put(dft, pft);
				if (dft.getItemType() == itemType && !existing.contains(dft))
				{
					members.add(new DataFile(dft, null, null));
				}
			}
			catch (Throwable t)
			{}
		}
	}
	
	// Load subtype-related file types
	if (subtype != null)
	{
		ItemQuery<ItemSubtypeFileType> query = subtype.getDataFileTypes();
		for (ItemSubtypeFileType sft : query.list(dc))
		{
			DataFileType dft = null;
			try
			{
				dft = sft.getDataFileType();
				usableFileTypes.put(dft, sft);
				if (dft.getItemType() == itemType && !existing.contains(dft))
				{
					members.add(new DataFile(dft, null, null));
				}
			}
			catch (Throwable t)
			{}
		}
	}
	
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, new GuiContext(itemType, GuiContext.Type.ITEM), null);
	FileViewerContext fileContext = FileViewerContext.dataFile((BasicItem)item);
	FileViewerUtil fileViewer = FileViewerUtil.getOrCreate(jspContext, fileContext);
	%>
	<base:page type="include">
	<base:body>
		<ext:scripts context="<%=jspContext%>" />
		<base:section 
			id="datafiles" 
			title="<%="Data files (" + (numMembers + numParents) +")"%>"
			context="<%=cc%>">
			<%
			if (members.size() == 0)
			{
				%>
				<div class="messagecontainer note">
				No files have been added to this item
				(or you don't have permission to view them).
				</div>
				<%
			}
			else
			{
				%>
				<tbl:table
					id="tbl.datafiles"
					columns="all"
					>
					<tbl:columndef 
						id="type"
						title="Type"
					/>
					<tbl:columndef 
						id="file"
						title="File"
					/>
					<tbl:columndef 
						id="required"
						title="Required"
					/>
					<tbl:columndef 
						id="validation"
						title="Validation"
					/>
					<tbl:data>
						<tbl:headers>
							<tbl:headerrow>
								<tbl:columnheaders />
							</tbl:headerrow>
							<tbl:columnsubtitles />
						</tbl:headers>
						<tbl:rows>
						<%
						for (DataFile dataFile : members)
						{
							FileSetMember member = dataFile.member;
							DataFileType dft = dataFile.type;
							FileStoreEnabled parent = dataFile.parent;
							Boolean validFile = null;
							String validationMessage = "";
							boolean readFile = true;
							File file = null;
							boolean readDataFileType = true;
							if (member != null)
							{
								try
								{
									file = member.getFile();
								}
								catch (PermissionDeniedException ex)
								{
									readFile = false;
								}
								try
								{
									dft = member.getDataFileType();
								}
								catch (PermissionDeniedException ex)
								{
									readDataFileType = false;
								}
							}
							UsableDataFileType uft = usableFileTypes.get(dft);
							boolean isRequired = uft == null ? false : uft.isRequired();
							boolean isPartOfPlatform = uft != null || (subtype != null && subtype.isAssociatedDataFileType(dft));
							String icon = null;
							if (member == null)
							{
								if (isRequired)
								{
									icon = "warning.png";
									validationMessage = "Missing a required file";
								}
							}
							else 
							{
								validFile = member.isValid();
								if (Boolean.FALSE.equals(validFile))
								{
									validationMessage = HTML.encodeTags(member.getErrorMessage());
									icon = "error.png";
								}
								else if (!isPartOfPlatform)
								{
									icon = "warning.png";
									if (platform != null)
									{
										validationMessage = "This file is not part of the <i>" + HTML.encodeTags(platform.getName()) + "</i> platform";
									}
									else if (subtype != null)
									{
										validationMessage = "This file is not part of the <i>" + HTML.encodeTags(subtype.getName()) + "</i> subtype";
									}
								}
								else if (Boolean.TRUE.equals(validFile))
								{
									if (member.getErrorMessage() != null)
									{
										validationMessage = HTML.encodeTags(member.getErrorMessage());
										icon = "warning.png";
									}
									else
									{
										validationMessage = "Ok";
										icon = "ok.png";
									}
								}
								else
								{
									validationMessage = "<i>- not validated -</i>";
								}
							}
							%>
							<tbl:row>
								<tbl:cell column="type"><base:icon 
										image="deleted.png" 
										tooltip="This item has been scheduled for deletion" 
										visible="<%=dft != null && dft.isRemoved()%>"
									/><%=Base.getLinkedName(ID, dft, !readDataFileType, true)%></tbl:cell>
								<tbl:cell column="file"><base:icon 
										image="deleted.png" 
										tooltip="This item has been scheduled for deletion" 
										visible="<%=file != null && file.isRemoved()%>"
									/>
									<%=Base.getLinkedName(ID, file, !readFile, true)%>
									<%
									if (file != null)
									{
										fileContext.setCurrentDataFile(file, dft, member);
										fileViewer.render();
									}
									if (parent != null)
									{
										%>
										<i>(inherited from <%=Base.getLinkedName(ID, (Nameable)parent, false, true)%>)</i>
										<%
									}
									%>
								</tbl:cell>
								<tbl:cell column="required"><%=isRequired ? "yes" : "no" %></tbl:cell>
								<tbl:cell column="validation">
									<base:icon image="<%=icon%>" visible="<%=icon != null %>" /> 
									<%=validationMessage %>
								</tbl:cell>
							</tbl:row>
							<%
						}
						%>
						</tbl:rows>
					</tbl:data>
				</tbl:table>
				<%
			}
			%>
		</base:section>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}

%>
