<%-- $Id: edit_batch.jsp 7954 2021-05-12 13:27:22Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------


	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.ArrayBatch"
	import="net.sf.basedb.core.ArrayDesign"
	import="net.sf.basedb.core.Protocol"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.Project"
	import="net.sf.basedb.core.Hardware"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.BaseException"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.edit.EditUtil"
	import="net.sf.basedb.clients.web.extensions.tabcontrol.TabAction"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="java.util.List"
	import="java.util.Set"
	import="java.util.HashSet"
	import="java.util.Date"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%
final Item itemType = Item.ARRAYBATCH;
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl(":Edit "+itemType);
try
{
	String title = null;
	ArrayBatch batch = null;
	
	boolean readCurrentArrayDesign = true;
	ArrayDesign currentArrayDesign = null;
	List<ArrayDesign> defaultArrayDesigns = null;
	boolean readCurrentProtocol = true;
	Protocol currentProtocol = null;
	List<Protocol> defaultProtocols = null;
	boolean readCurrentPrintRobot = true;
	Hardware currentPrintRobot = null;
	List<Hardware> defaultPrintRobots = null;

	// Load recently used items
	List<ArrayDesign> recentArrayDesigns = cc.getRecent(dc, Item.ARRAYDESIGN);
	List<Protocol> recentProtocols = cc.getRecent(dc, Item.PROTOCOL);
	List<Hardware> recentPrintRobots = cc.getRecent(dc, Item.HARDWARE);
	
	int activeProjectId = sc.getActiveProjectId();
	if (activeProjectId > 0)
	{
		Project activeProject = Project.getById(dc, activeProjectId);
		try
		{
			defaultArrayDesigns = activeProject.findDefaultItems(dc, Item.ARRAYDESIGN, true);
		}
		catch (PermissionDeniedException pdex)
		{}
		try
		{
			defaultProtocols = activeProject.findDefaultItems(dc, 
					ItemSubtype.getById(dc, SystemItems.getId(Protocol.PRINTING)), false);
		}
		catch (PermissionDeniedException pdex)
		{}
		try
		{
			defaultPrintRobots = activeProject.findDefaultItems(dc, 
					ItemSubtype.getById(dc, SystemItems.getId(Hardware.PRINT_ROBOT)), false);
		}
		catch (PermissionDeniedException pdex)
		{}
	}
	if (itemId == 0)
	{
		title = "Create array batch";
		cc.removeObject("item");
		int arrayDesignId = Values.getInt(request.getParameter("arraydesign_id"));
		if (arrayDesignId != 0)
		{
			currentArrayDesign = ArrayDesign.getById(dc, arrayDesignId);
		}
		else if (cc.getPropertyFilter("arrayDesign.name") != null)
		{
			currentArrayDesign = Base.getFirstMatching(dc, ArrayDesign.getQuery(), "name", cc.getPropertyFilter("arrayDesign.name"));
		}
		if (cc.getPropertyFilter("printRobot.name") != null)
		{
			currentPrintRobot = Base.getFirstMatching(dc, Hardware.getQuery(), "name", cc.getPropertyFilter("printRobot.name"));
		}
		if (cc.getPropertyFilter("protocol.name") != null)
		{
			currentProtocol = Base.getFirstMatching(dc, Protocol.getQuery(), "name", cc.getPropertyFilter("protocol.name"));
		}
	}
	else
	{
		batch = ArrayBatch.getById(dc, itemId);
		batch.checkPermission(Permission.WRITE);
		cc.setObject("item", batch);
		title = "Edit array batch -- " + HTML.encodeTags(batch.getName());
		try
		{
			currentArrayDesign = batch.getArrayDesign();
		}
		catch (PermissionDeniedException ex)
		{
			readCurrentArrayDesign = false;
		}
		try
		{
			currentPrintRobot = batch.getPrintRobot();			
		}
		catch (PermissionDeniedException ex)
		{
			readCurrentPrintRobot = false;
		}
		try
		{
			currentProtocol = batch.getProtocol();			
		}
		catch (PermissionDeniedException ex)
		{
			readCurrentProtocol = false;
		}
	}	
	
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, GuiContext.item(itemType), batch);
	ExtensionsInvoker<TabAction> invoker = EditUtil.useEditExtensions(jspContext);
	%>
	<base:page type="popup" title="<%=title%>" id="edit-page">
	<base:head scripts="tabcontrol-2.js,~batches.js" styles="tabcontrol.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<h1><%=title%> <base:help tabcontrol="settings" /></h1>
		<div id="page-data" class="datacontainer"
			data-printing-protocol="<%=SystemItems.getId(Protocol.PRINTING)%>"
			data-print-robot="<%=SystemItems.getId(Hardware.PRINT_ROBOT)%>"
		></div>
		
		<form action="index.jsp?ID=<%=ID%>" method="post" name="batch">
		<input type="hidden" name="cmd" value="UpdateItem">

		<t:tabcontrol id="settings" 
			subclass="content dialogtabcontrol"
			position="bottom" remember="<%=batch != null%>" 
			extensions="<%=invoker%>">
		<t:tab id="info" title="Array batch" helpid="arraybatch.edit">
			<table class="fullform input100">
			<tr>
				<th>Name</th>
				<td><input class="text required auto-init" data-auto-init="<%=batch == null ? "focus-select" : "focus" %>"
					type="text" name="name" 
					value="<%=HTML.encodeTags(batch == null ? Values.getString(cc.getPropertyValue("name"), "New array batch") : batch.getName())%>" 
					maxlength="<%=ArrayBatch.MAX_NAME_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>Array design</th>
				<td>
					<%
					if (batch != null)
					{
						%>
						<%=Base.getEncodedName(currentArrayDesign, !readCurrentArrayDesign)%>
						<%
					}
					else
					{
						%>
						<base:select 
							id="arraydesign_id"
							clazz="selectionlist required"
							required="true"
							current="<%=currentArrayDesign%>"
							denied="<%=!readCurrentArrayDesign%>"
							recent="<%=recentArrayDesigns%>"
							defaultitems="<%=defaultArrayDesigns%>"
							newitem="true"
						/>
						<%
					}
					%>
				</td>
				<td></td>
			</tr>
			<tr>
				<th>Print robot</th>
				<td>
					<base:select 
						id="printrobot_id"
						clazz="selectionlist"
						required="false"
						current="<%=currentPrintRobot%>"
						denied="<%=!readCurrentPrintRobot%>"
						recent="<%=recentPrintRobots%>"
						defaultitems="<%=defaultPrintRobots%>"
						newitem="<%=batch == null%>"
					/>
				</td>
				<td></td>
			</tr>
			<tr>
				<th>Protocol</th>
				<td>
					<base:select 
						id="protocol_id"
						clazz="selectionlist"
						required="false"
						current="<%=currentProtocol%>"
						denied="<%=!readCurrentProtocol%>"
						recent="<%=recentProtocols%>"
						defaultitems="<%=defaultProtocols%>"
						newitem="<%=batch == null%>"
					/>
				</td>
				<td></td>
			</tr>
			<tr class="dynamic">
				<th>Description</th>
				<td>
					<textarea class="text" rows="6" name="description" id="description"
						><%=HTML.encodeTags(batch == null ? cc.getPropertyValue("description") : batch.getDescription())%></textarea>
				</td>
				<td style="width: 20px;">
					<base:zoom textarea="description" title="Description" />
				</td>
			</tr>
			</table>
		</t:tab>
		
		<t:tab id="annotations" title="Annotations &amp; parameters" helpid="annotations.edit">
			<jsp:include page="../../common/annotations/annotate_frameset.jsp">
				<jsp:param name="item_type" value="<%=itemType.name()%>" />
				<jsp:param name="item_id" value="<%=itemId%>" />
				<jsp:param name="ID" value="<%=ID%>" />
			</jsp:include>
		</t:tab>
		</t:tabcontrol>
		</form>

		<div class="legend">
			<base:icon image="required.png" />= required information
		</div>

		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnSave" title="Save" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
